import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;
import java.util.StringTokenizer;

import scg.ac.ni.pmf.newgraph.graphdata.DuplicateIDException;
import scg.ac.ni.pmf.newgraph.graphdata.DuplicateLabelException;
import scg.ac.ni.pmf.newgraph.graphdata.EdgeData;
import scg.ac.ni.pmf.newgraph.graphdata.Graph;
import scg.ac.ni.pmf.newgraph.graphdata.GraphData;
import scg.ac.ni.pmf.newgraph.graphdata.VertexData;
import scg.ac.ni.pmf.newgraph.io.GraphFileType;
import scg.ac.ni.pmf.newgraph.io.GraphRWException;

public class GraphTxtFileType implements GraphFileType {
	
	public String getName() {
		return "Graph Txt";
	}

	public String getExtension() {
		return "GraphTxt";
	}
	
	public boolean accepts(File file) {
		return file.getName().endsWith(".GraphTxt");
	}
	
	public void read(File file, Graph graph) throws GraphRWException {
		try {
			BufferedReader in = new BufferedReader(new FileReader(file));
			int n = Integer.parseInt(in.readLine());
			for(int i = 0; i < n; i++) {
				StringTokenizer st = new StringTokenizer(in.readLine(), ", ");
				double x = Double.parseDouble(st.nextToken());
				double y = Double.parseDouble(st.nextToken());
				int id = Integer.parseInt(st.nextToken());
				String label = st.nextToken();
				try {
					graph.createVertex(id, x, y, label);
				} catch(DuplicateIDException e) {
				} catch(DuplicateLabelException e) {
				}
			}
			
			int m = Integer.parseInt(in.readLine());
			for(int i = 0; i < m; i++) {
				StringTokenizer st = new StringTokenizer(in.readLine(), ", ");
				int idA = Integer.parseInt(st.nextToken());
				int idB = Integer.parseInt(st.nextToken());
				graph.createEdge(idA, idB);
			}
		} catch(IOException e) {
			throw new GraphRWException(e.getMessage());
		} catch(NumberFormatException e) {
			throw new GraphRWException(e.getMessage());
		}
	}

	public void write(File file, GraphData data) throws GraphRWException {
		try {
			PrintWriter out = new PrintWriter(new FileWriter(file));
			out.println(data.getOrder());
			for(Enumeration i = data.vertices(); i.hasMoreElements(); ) {
				VertexData vertex = (VertexData) i.nextElement();
				out.println(vertex.getX() + ", " + vertex.getY() + ", " + vertex.getID() +
						", " + vertex.getLabel());
			}
			out.println(data.getNumberOfEdges());
			for(Enumeration i = data.edges(); i.hasMoreElements(); ) {
				EdgeData edge = (EdgeData) i.nextElement();
				out.println(edge.getIDA() + ", " + edge.getIDB());
			}
			out.close();
		} catch(IOException e) {
			throw new GraphRWException(e.getMessage());
		}
	}

}