import cern.colt.matrix.*;
import cern.colt.matrix.impl.*;
import cern.colt.matrix.linalg.*;

import scg.ac.ni.pmf.newgraph.graphdata.GraphData;
import scg.ac.ni.pmf.newgraph.graphdata.VertexData;
import scg.ac.ni.pmf.newgraph.invariants.InvariantDoubleVertex;
import scg.ac.ni.pmf.newgraph.invariants.VertexDoubleValues;
import java.util.Enumeration;

public class PE implements InvariantDoubleVertex {
        public String getName() {
                return "PE";
        }

        public String getDescription() {
                return "The principle eigenvector (to be visualized).";
        }
        
        public void calculate(GraphData data, VertexDoubleValues values) {
             // calculate the principal eigenvector
             double[] peArray = principalEigenvector(data);

             // ensure principal eigenvector is positive!
             int n = data.getOrder();
             for (int k=0; k<n; k++)
                 if (peArray[k]<0)
                     peArray[k] = -peArray[k];

             // assign the values to vertices
             int k=0;
             for (Enumeration i = data.vertices(); i.hasMoreElements(); ) {
                 VertexData vertex = (VertexData) i.nextElement();
                 values.assignValue(vertex, peArray[k]);
                 k++;
             }
        }
        
        private double[] principalEigenvector(GraphData data) {
                int n = data.getOrder();
                int[][] adj = data.adjacencyMatrix();
                double[][] dadj = new double[n][n];
 
                // form a generalized adjacency matrix
                for (int i=0; i<n; i++)
                  for (int j=0; j<n; j++)
                     dadj[i][j]=adj[i][j];
        
                // find the eigenvectors
                DoubleMatrix2D cernAdj = new DenseDoubleMatrix2D(dadj);
                EigenvalueDecomposition cernDecomposition = new EigenvalueDecomposition(cernAdj);
                DoubleMatrix2D genV = cernDecomposition.getV();
                double[][] eigenvectors = genV.toArray();
                
                double[] pe = new double[n];
                for (int i=0; i<n; i++)
                    pe[i] = eigenvectors[i][0];
                return pe;
        }
}

