import cern.colt.matrix.*;
import cern.colt.matrix.impl.*;
import cern.colt.matrix.linalg.*;

import scg.ac.ni.pmf.newgraph.graphdata.GraphData;
import scg.ac.ni.pmf.newgraph.graphdata.VertexData;
import scg.ac.ni.pmf.newgraph.invariants.InvariantString;
import scg.ac.ni.pmf.newgraph.param.Parametrizable;

public class MaxBipAllPairs implements InvariantString, Parametrizable {
        public String getName() {
                return "MaxBip Subgraph (all pairs)";
        }

        public String getDescription() {
                return "Maximum bipartite subgraph from all pairs of eigenvectors.";
        }
        
        public String calculate(GraphData data) {
                int n = data.getOrder();
                int[][] adj = data.adjacencyMatrix();
                double[][] dbadj = new double[n][n];
                
                // copy to a double[][] matrix
                for (int i=0; i<n; i++)
                    for (int j=0; j<n; j++)
                        dbadj[i][j] = (double)adj[i][j];
 
                // find a spectrum and eigenvectors
                DoubleMatrix2D cernAdj = new DenseDoubleMatrix2D(dbadj);
                EigenvalueDecomposition cernDecomposition = new EigenvalueDecomposition(cernAdj);
                DoubleMatrix1D spectrum = cernDecomposition.getRealEigenvalues();
                DoubleMatrix2D eigenV = cernDecomposition.getV();
                // note that eigenvectors are placed in COLUMNS!
                
                // test all pairs of eigenvectors
                int maxbipsubedges = 0;
                int[] maxSignVertex = new int[n];
                                
                for (int i=0; i<n; i++)
                for (int j=i+1; j<n; j++)
                {   // divide the vertices according to the Hadamard product of eigenvectors
                    int[] signVertex = new int[n];
                    for (int k=0; k<n; k++)
                        if (eigenV.get(k,i)*eigenV.get(k,j)<beta)
                            signVertex[k]=-1;
                        else
                            signVertex[k]=1;
                    // how many edges in such bipartite subgraph?
                    int bipsubedges=0;
                    for (int k=0; k<n; k++)
                        for (int l=k+1; l<n; l++)
                            if ((signVertex[k]*signVertex[l]==-1)
                                 && (adj[k][l]==1))
                                bipsubedges++;
                    // a new record?
                    if (bipsubedges > maxbipsubedges)
                    {
                        maxbipsubedges = bipsubedges;
                        for (int k=0; k<n; k++)
                            maxSignVertex[k] = signVertex[k];
                    }
                }

                // form the result string
                String rezultat = "" + maxbipsubedges + ", part I: ";
                for (int k=0; k<n; k++)
                    if (maxSignVertex[k]==1)
                        rezultat = rezultat + k + ", ";
                rezultat = rezultat + "part II: ";
                for (int k=0; k<n; k++)
                    if (maxSignVertex[k]==-1)
                        rezultat = rezultat + k + ", ";
                return rezultat;
        }
        
        private double beta=0;

        public double getBeta() {
                return beta;
        }

        public void setBeta(double beta) {
                this.beta = beta;
        }

        public String checkParameters() {
                return null;
        }

}
